***********************************
* Feito por Jos Quintas          *
* Modificado por Anderson Cardoso *
***********************************
// Modificaes incluem informaes adicionais com dbinfo.ch
// Traduo para portugus, nova diagramao.
***********************************

#include "error.ch"
#include "hbgtinfo.ch"
#include "dbinfo.ch"

// put messages to STDERR
#command ?    =>  ?? hb_Eol() ; ?? 
#command ??   =>  OutErr()

* Note:  automatically executes at startup

PROCEDURE ERRORSYS

   ErrorBlock( { | e | JoseQuintasError( e ) } )

   RETURN

STATIC FUNCTION JoseQuintasError( e )

   LOCAL nCont, cMessage

   // by default, division by zero yields zero
   IF ( e:GenCode == EG_ZERODIV )
      RETURN 0
   ENDIF

   // build error message
   cMessage := ErrorMessage(e)
   hb_Default( @cMessage )
   IF ! Empty( e:OsCode )
      cMessage += ";(DOS Error " + Ltrim( Str( e:OsCode ) ) + ")"
   ENDIF
   IF cMessage == NIL
      cMessage := ""
   ENDIF

   // Only retry IF open error 2014.09.24.1810
   IF e:OsCode == 64 .AND. e:GenCode == EG_OPEN
      //@ 15, 15 SAY "Servidor sumiu. Tentar novamente em 2 segundos"
      Inkey(5)
      RETURN .T.
   ENDIF

   // For network open error, set NetErr() and subsystem default
   IF ( e:GenCode == EG_OPEN .AND. e:OsCode == 32 .AND. e:CanDefault )
      NetErr( .T. )
      RETURN ( .F. )     // NOTE
   ENDIF

   // for lock error during APPEND BLANK, set NetErr() and subsystem default
   IF ( e:GenCode == EG_APPENDLOCK .AND. e:CanDefault )
      NetErr( .T. )
      RETURN ( .F. )     // NOTE
   ENDIF
   Errorsys_WriteErrorLog( "----------- ERRO NO SISTEMA", 1 ) // com id maquina
   @ MaxRow(), 0 SAY ""
   ? cMessage
   Errorsys_WriteErrorLog( cMessage )
   nCont := 2
   DO WHILE ( ! Empty( ProcName( nCont ) ) )
      cMessage := "Called from " + Trim( ProcName( nCont ) ) + "(" + Ltrim( Str( ProcLine( nCont ) ) ) + ")  "
      ? cMessage
      Errorsys_WriteErrorLog( cMessage )
      nCont++
   ENDDO
   Errorsys_WriteErrorLog( Replicate( "-", 80 ) )
   RUN ( "start notepad.exe hb_out.log" )
   // give up
   ErrorLevel( 1 )
   QUIT

   RETURN .F.

STATIC FUNCTION ErrorMessage( e )

   LOCAL cMessage := ""

   // start error message
   BEGIN SEQUENCE WITH __BreakBlock()
      cMessage := if( e:Severity > ES_WARNING, "Erro ", "Aviso " )
   ENDSEQUENCE

   // add subsystem name IF available
   IF ( ValType( e:SubSystem ) == "C" )
      cMessage += e:SubSystem
   ELSE
      cMessage += "???"
   ENDIF

   // add subsystem's error code IF available
   IF ( ValType( e:SubCode ) == "N" )
      cMessage += ( "/" + Ltrim(Str( e:SubCode ) ) )
   ELSE
      cMessage += "/???"
   ENDIF

   // add error description IF available
   IF ( ValType( e:Description ) == "C" )
      cMessage += ( ": " + e:Description )
   ENDIF

   // add either filename or operation
   IF ! Empty( e:Filename )
      cMessage += (": " + e:Filename )
   ELSEIF ! Empty( e:Operation )
      cMessage += ( ": " + e:Operation )
   ENDIF

   RETURN cMessage

FUNCTION Errorsys_WriteErrorLog( cText, nDetail )

   LOCAL hFileOutput, cFileName, nCont, nCont2

   hb_Default( @cText, "" )
   hb_Default( @nDetail, 0 )

   IF nDetail > 0
		Errorsys_WriteErrorLog()
		Errorsys_WriteErrorLog( "Ocorrido em: " + Dtoc( Date() ) + " s " + Time() )
		Errorsys_WriteErrorLog( "Nome do EXE: " + hb_Argv(0) )
		Errorsys_WriteErrorLog()
		Errorsys_WriteErrorLog( "Banco de dados...........: " + IIF( !EMPTY(Alias()),   Alias(),   "Nenhum aberto") )
		Errorsys_WriteErrorLog( "Indice...................: " + IIF( !EMPTY(OrdName()), OrdName(), "Nenhum aberto") )
		IF !EMPTY(OrdName())
			// SE NO TEM NEM NDICE ABERTO NO TEM PRA QU ISSO
			Errorsys_WriteErrorLog( "Campo classificado.......: " + IIF( !EMPTY(OrdKey()),  OrdKey(),  "Nenhum") )
			Errorsys_WriteErrorLog( "Condio de classificao: " + IIF( !EMPTY(OrdFor()),  OrdFor(),  "Nenhuma") )
		ENDIF		
		Errorsys_WriteErrorLog()
		Errorsys_WriteErrorLog( "RDD..................: " + RddSetDefault() )
		Errorsys_WriteErrorLog( "Extenso DBF.........: " + hb_rddInfo( RDDI_TABLEEXT ) )
		Errorsys_WriteErrorLog( "Extenso dos ndices.: " + IndexExt() )
		Errorsys_WriteErrorLog( "Extenso do Memo.....: " + hb_rddInfo( RDDI_MEMOEXT  ) )		
		Errorsys_WriteErrorLog( "Cdigo de Pgina.....: " + HB_SetCodePage()            ) // hb_rddInfo( DBI_CODEPAGE  ) )		
		Errorsys_WriteErrorLog()
		Errorsys_WriteErrorLog( "Diretrio.........: " + hb_cwd() )
		Errorsys_WriteErrorLog( "Verso do Windows.: " + OS() )
		Errorsys_WriteErrorLog( "Nome do Computador: " + GetEnv( "COMPUTERNAME" ) )
		Errorsys_WriteErrorLog( "Usurio do Windows: " + GetEnv( "USERNAME" ) )
		Errorsys_WriteErrorLog( "Servidor..........: " + Substr( GetEnv( "LOGONSERVER" ), 2 ) )
		Errorsys_WriteErrorLog( "Domnio do usurio: " + GetEnv( "USERDOMAIN" ) )
		Errorsys_WriteErrorLog()
		Errorsys_WriteErrorLog( "Verso do Harbour...: " + Version() )
		Errorsys_WriteErrorLog( "Verso do Compilador: " + HB_Compiler() )
		Errorsys_WriteErrorLog( "Biblioteca GT.......: " + hb_GtInfo( HB_GTI_VERSION ) )
		Errorsys_WriteErrorLog()
   ENDIF
   cFileName := "hb_out.log"
   IF ! File( cFileName )
      hFileOutput := fCREATE( cFileName )
      fClose( hFileOutput )
   ENDIF

   hFileOutput := fOpen( cFileName, 1 )
   fSeek( hFileOutput, 0, 2 )
   cText := HB_OEMTOANSI(cTEXT)
   fWrite( hFileOutput, cText + Space(2) + hb_Eol() )
   IF nDetail > 1
      nCont  := 1
      nCont2 := 0
      DO WHILE nCont2 < 5
         IF Empty( ProcName( nCont ) )
            nCont2++
         ELSE
            cText := "Called from " + Trim( ProcName( nCont ) ) + "(" + Ltrim( Str( ProcLine( nCont ) ) ) + ")  "
            fWrite( hFileOutput, cText + hb_Eol() )
         ENDIF
         nCont++
      ENDDO
      fWrite( hFileOutput, hb_Eol() )
   ENDIF
   fClose( hFileOutput )

   RETURN NIL